#include <Adafruit_NeoPixel.h>
#include <Adafruit_CircuitPlayground.h>

int strikeTarget, strikedLocation;

int capThreshold[6];  // Array to store touch threshold values
int capOffset = 300;

int capPins[] = { A1, A2, A3, A4, A5, A6 };  // Capacitive touch pins
bool IsStrikedRight = false;
bool IsStriked = false;

int targetTime, hitTime;

#define LED_PIN_Right 1  // Data pin for the Right LED strip
#define LED_PIN_Left 12    // Data pin for the Left LED strip
#define NUMPIXELS 30      // Total number of LEDs on the strip
Adafruit_NeoPixel stripRight = Adafruit_NeoPixel(NUMPIXELS, LED_PIN_Right, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel stripLeft = Adafruit_NeoPixel(NUMPIXELS, LED_PIN_Left, NEO_GRB + NEO_KHZ800);


#define NOTE_C4 262
#define NOTE_D4 294
#define NOTE_E4 330
#define NOTE_F4 349
#define NOTE_G4 392
#define NOTE_A4 440
#define NOTE_B4 494
#define NOTE_C5 523

void setup() {
  Serial.begin(9600);
  CircuitPlayground.begin();
  stripRight.begin();
  //stripRight.setBrightness(10);  // Adjust brightness (0-255)
  stripLeft.begin();
  //stripRight.setBrightness(10);  // Adjust brightness (0-255)

  stripRight.clear();
  stripLeft.clear();
  stripRight.show();
  stripLeft.show();

  randomSeed(analogRead(12));
  INZfunction();
  TGNfunction();
  strikedLocation = -1;
}

void loop() {
  if (IsStrikedRight == true) {
    TGNfunction();
  }

  INPfunction();
  OTPfunction();

  delay(10);
}

//LED Manage Function
void LEDfunction(int location) {
  stripRight.clear();
  stripLeft.clear();
  stripRight.show();
  stripLeft.show();
  Serial.println(location);
  if (location == 0) {
    for (int i = 0; i < 10; i++) {
      LEDRight(i);
    }
  } else if (location == 3) {
    for (int i = 0; i < 10; i++) {
      LEDLeft(i);
    }
  } else if (location == 1) {
    for (int i = 10; i < 20; i++) {
      LEDRight(i);
    }
  } else if (location == 4) {
    for (int i = 10; i < 20; i++) {
      LEDLeft(i);
    }
  } else if (location == 2) {
    for (int i = 20; i < 30; i++) {
      LEDRight(i);
    }
  } else if (location == 5) {
    for (int i = 20; i < 30; i++) {
      LEDLeft(i);
    }
  }
}

void LEDRight(int LEDLocation) {
  stripLeft.clear();
  if (IsStrikedRight) {
    stripRight.setPixelColor(LEDLocation, stripRight.Color(0, 255, 0));
  } else {
    stripRight.setPixelColor(LEDLocation, stripRight.Color(255, 0, 0));
  }
  stripRight.show();  // Refresh the LED strip to apply changes
}
void LEDLeft(int LEDLocation) {
  stripRight.clear();
  if (IsStrikedRight) {
    stripLeft.setPixelColor(LEDLocation, stripLeft.Color(0, 255, 0));
  } else {
    stripLeft.setPixelColor(LEDLocation, stripLeft.Color(255, 0, 0));
  }
  stripLeft.show();  // Refresh the LED strip to apply changes
}

// Target Generation Function
void TGNfunction() {
  strikeTarget = random(0, 6);
  Serial.print("strike target:");
  Serial.println(strikeTarget + 1);
  tunePlayer(1);
  IsStrikedRight = false;
  LEDfunction(strikeTarget);
  targetTime = millis();
}

// Input processing Function
void INPfunction() {
  int currentLocation = strikedLocation;
  for (int i = 0; i < 6; i++) {
    int capValue = CircuitPlayground.readCap(capPins[i]);

    if (capValue > capThreshold[i]) {
      strikedLocation = i;
      IsStriked = true;
      if (strikedLocation != currentLocation) {
        Serial.println(strikedLocation);
      }
    }
  }
}

// Output processing Function
void OTPfunction() {
  if (IsStriked) {
    if (strikeTarget == strikedLocation) {
      hitTime = millis();
      Serial.print("strike time:");
      Serial.print(hitTime - targetTime);
      Serial.println(" millisecond");
      strikedLocation = -1;
      IsStrikedRight = true;
      LEDfunction(strikeTarget);
      tunePlayer(2);
      delay(500);
    } else if (strikeTarget != strikedLocation) {
      tunePlayer(3);
    }
    IsStriked = false;
  }
}

// Initializing Function
void INZfunction() {
  delay(100);
  for (int i = 0; i < 6; i++) {
    int sum = 0;

    for (int j = 0; j < 20; j++) {
      sum += CircuitPlayground.readCap(capPins[i]);
      delay(50);
    }

    capThreshold[i] = (sum / 20) + capOffset;  // calculate the average and add a offset
  }
  printThresholds();
  tunePlayer(0);  //play starting tune
}

void printThresholds() {
  Serial.print("Capacitive Touch Thresholds: ");

  for (int i = 0; i < 6; i++) {  // Loop through the array
    Serial.print(capThreshold[i]);
    Serial.print(" ");  // Print space between values
  }

  Serial.println();  // Move to the next line
}

void tunePlayer(int pattern) {

  //starting
  if (pattern == 0) {
    int melody[] = { NOTE_C4, NOTE_E4, NOTE_G4, NOTE_C5, NOTE_G4, NOTE_C5 };
    int durations[] = { 50, 50, 50, 100, 50, 150 };  // Durations in ms
    for (int i = 0; i < 6; i++) {
      CircuitPlayground.playTone(melody[i], durations[i]);
    }
  }
  //target show up
  if (pattern == 1) {
    int melody[] = { NOTE_C5 };
    int durations[] = { 150 };  // Durations in ms
    for (int i = 0; i < 1; i++) {
      CircuitPlayground.playTone(melody[i], durations[i]);
    }
  }
  //right target strike
  if (pattern == 2) {
    int melody[] = { NOTE_C5, NOTE_C5 };
    int durations[] = { 50, 250 };  // Durations in ms
    for (int i = 0; i < 2; i++) {
      CircuitPlayground.playTone(melody[i], durations[i]);
    }
  }
  //wrong target strike
  if (pattern == 3) {
    int melody[] = { NOTE_C4 };
    int durations[] = { 50 };  // Durations in ms
    for (int i = 0; i < 1; i++) {
      CircuitPlayground.playTone(melody[i], durations[i]);
    }
  }
}
